//
// Dan Bricklin's Mobile Test Utility
//

/////////////////////////////////////////////
/*

SOFTWARE LICENSE

This software and documentation is
Copyright (c) 2009 Software Garden, Inc.
All rights reserved. 

1. The source code of this program is made available as free software;
you can redistribute it and/or modify it under the terms of the GNU
General Public License, version 2, as published by the Free Software
Foundation.

2. This program is distributed in the hope that it will be useful, but
WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
General Public License for more details. You should have received a
copy of the GNU General Public License along with this program; if not,
write to the Free Software Foundation, Inc., 59 Temple Place - Suite
330, Boston, MA  02111-1307, USA.

3. If the GNU General Public License is restrictive in a way that does
not meet your needs, contact the copyright holder (Software Garden,
Inc.) to inquire about the availability of other licenses, such as
traditional commercial licenses. 

4. The right to distribute this software or to use it for any purpose
does not give you the right to use Servicemarks or Trademarks of
Software Garden, Inc., including Garden, Software Garden, and
ListGarden, nor the Servicemarks or Trademarks of Daniel S. Bricklin, 
including Dan Bricklin's. If you make modifications to this software 
you must remove displayed references to the Dan Bricklin's trademark.

5. An appropriate copyright notice will include the Software Garden,
Inc., copyright, and a prominent change notice will include a
reference to Software Garden, Inc., as the originator of the code
to which the changes were made.

Contact and other information about Software Garden, Inc., may be 
found on the Software Garden web site: http://www.softwaregarden.com.

Software Garden is a registered trademark of Software Garden, Inc.
Dan Bricklin's is a registered trademark of Daniel S. Bricklin

*/

/////////////////////////////////////////////
//
// HISTORY:
//
// Initially coded by Dan Bricklin, July 2009, for Software Garden, Inc.
//
// Version: 1.0 - 29 July 2009
//
//
// HOW TO USE THIS CODE:
//
// You can invoke this code using an HTML file such as the following:
//
/*
   <!DOCTYPE HTML PUBLIC "-//W3C//DTD HTML 4.01//EN" "http://www.w3.org/TR/html4/strict.dtd">
   <HTML>
   <HEAD>
   <META http-equiv="Content-Type" content="text/html; charset=UTF-8">
   <title>Dan Bricklin's&reg; Mobile Test Utility 1.0</title>
   <script type="text/javascript" src="dbmobiletest.js"></script>

   <style>
   body
    {font-family:verdana,helvetica,sans-serif; font-size:small;}
   .superscript
    {vertical-align:super;font-size:smaller;}
   </style>

   </HEAD>
   <BODY onload="SetupTests();">
   <div id="mainele">
   </div>
   </BODY>

*/

/////////////////////////////////////////////
//
// START OF CODE
//
/////////////////////////////////////////////

var MobileTestVersion = "1.0";

function SetupTests() {

   var str, mainele, ele, i;

   mainele = GEBI("mainele");
   if (!mainele) {
      alert("No mainele element. Cannot run tests.");
      }

   window.onerror = ErrorHandler;

   str = '<select id="testtype" onchange="ShowMore();">';
   for (i=0; i<Tests.length; i++) {
      str += '<option value="' + i + '">' + Tests[i].name + '</option>';
      }
   str += '</select>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<input type="button" value="Run Test" onclick="RunTest();"><br>';

   str += '<div id="moreinfo" style="width:300px;padding:6px 0 12px 0;">&nbsp;</div>'+
'<div id="testresults">'+
'</div>';

   mainele.innerHTML = str;

   ShowMore();

   }

var Tests = [
   {name: "List All Tests", handler: ListTests, args: null,
      more: "Displays a list of all the tests along with a short description of what they do."},
   {name: "List Window Attributes", handler: ListObjectAttribs, args: {obj: window, name: "Window"},
      more: "Displays a sorted list of the attributes of the window object."},
   {name: "List Document Attributes", handler: ListObjectAttribs, args: {obj: document, name: "Document"},
      more: "Displays a sorted list of the attributes of the document object."},
   {name: "List DIV Attributes", handler: ListIDAttribs, args: {id: "mainele", name: "DIV Element"},
      more: "Displays a sorted list of the attributes of one of the DIV objects."},
   {name: "List Select Attributes", handler: ListIDAttribs, args: {id: "testtype", name: "Select Element"},
      more: "Displays a sorted list of the attributes of the select object above."},
   {name: "Scroll Test 1", handler: ScrollTest, args: {size: 500, button: 25, style: ""},
      more: "Displays a lot of text that you can scroll along with buttons that show various window properties at that point."},
   {name: "Scroll Test 2", handler: ScrollTest, args: {size: 500, button: 25, style: "width:300px;"},
      more: "Displays a lot of text in a fixed width DIV that you can scroll along with buttons that show various window properties at that point."},
   {name: "Scroll Test 3", handler: ScrollTest, args: {size: 500, button: 25, style: "height:600px;overflow:auto;"},
      more: "Displays a lot of text in a fixed height DIV that you can scroll along with buttons that show various window properties at that point."},
   {name: "Speed Test 10,000", handler: SpeedTest, args: {ntimes: 10000},
      more: "Performs a variety of common JavaScript operations in a loop and displays the total times."},
   {name: "Speed Test 100,000", handler: SpeedTest, args: {ntimes: 100000},
      more: "Performs a variety of common JavaScript operations in a loop and displays the total times."},
   {name: "Mouse Event Test", handler: EventTest, args: null,
      more: "Shows the response to a variety of mouse events."},
   {name: "[None]",
      more: "Has no test handler and should result in an error which may or may not be displayed."}
   ];

//
// RunTest() - runs the selected test
//

function RunTest() {

   var ttele = GEBI("testtype");
   var miele = GEBI("moreinfo");
   var trele = GEBI("testresults");

   var testinfo = Tests[ttele.selectedIndex];

   var str = '<div style="width:300px;border:1px solid black;padding:6px;text-align:center;">'+
'Running test:<br><b>' + testinfo.name + '</b>'+
'<div style="font-style:italic;font-size:smaller;">' + (testinfo.more || "") + '</div>'+
'</div>';

   miele.innerHTML = str;

   testinfo.handler(trele, testinfo.args);

   }

//
// ShowMore() - Display "more" text for test
//

function ShowMore() {

   var ttele = GEBI("testtype");
   var miele = GEBI("moreinfo");
   var trele = GEBI("testresults");

   var testinfo = Tests[ttele.selectedIndex];

   if (testinfo.more) {
      miele.innerHTML = "<b>" + testinfo.name + ":</b> <i>" + testinfo.more + "</i>";
      }
   else {
      miele.innerHTML = "&nbsp;";
      }

   var str =
'<div style="width:300px;border:1px solid black;padding:6px;">'+
'<div style="text-align:center;padding:0 0 10px 0;font-weight:bold">'+
'Dan Bricklin\'s<span class="superscript">&reg;</span><br>Mobile Test Utility'+
'</div>'+
'Choose the test you want to run above and then press the "Run Test" button.'+
'</div>'+
'<div style="padding:10px 0 0 0;font-size:smaller;">'+
'Version '+MobileTestVersion+'<br><br>'+
'&copy; Copyright 2009 Software Garden, Inc.<br>All Rights Reserved.<br>'+
'Dan Bricklin\'s is a registered trademark of Daniel S. Bricklin.<br><br>'+
'This is free software that you may redistibute under certain conditions.<br>'+
'It comes with ABSOLUTELY NO WARRANTY.<br>'+
'See license and warranty information in the JavaScript source code.<br>'+
'</div>';

   trele.innerHTML = str;

   }

//
// ListTests(trele, objinfo) - Display "more" text for all the tests
//

function ListTests(trele, objinfo) {

   var str, i;

   str = "";

   for (i=0; i<Tests.length; i++) {
      str += '<b>' + Tests[i].name + ':</b> <i>' + (Tests[i].more || "") + '</i><br><br>';
      }
   str += '';

   trele.innerHTML = str;

   }

//
// ListObjectAttribs(trele, objinfo) - shows attribs of the objinfo.obj (name is objinfo.name)
//

function ListObjectAttribs(trele, objinfo) {

   var attrib, str, oa, oastr;
   var attribs = [];

   str = 'Attributes of ' + objinfo.name + ' (sorted using "ArrayObject.sort()"):<br>';

   for (attrib in objinfo.obj) {
      try {
         oa = objinfo.obj[attrib];
         } catch (ex) {alert(ex+"\n"+attrib);}
      switch (typeof oa) {
         case "number":
            oastr = oa+"";
            break;
         case "string":
            oastr = "string("+oa.length+")";
            break;
         case "boolean":
            oastr = oa;
            break;
         default:
            oastr = typeof oa;
            break;
         }
      attribs.push(attrib+": "+oastr);
      }
   attribs.sort();
   str += attribs.join('<br>');

   trele.innerHTML = str;

   }

//
// ListIDAttribs(trele, objinfo) - shows attribs of the element objinfo.id (name is objinfo.name)
//

function ListIDAttribs(trele, objinfo) {

   ListObjectAttribs(trele, {obj: GEBI(objinfo.id), name: objinfo.name});

   }

//
// ScrollTest(trele, args) - Makes a large div and shows values.
// Size is determined by args.size, buttons put in every args.button, div style is args.style.
//

function ScrollTest(trele, args) {

   var str, i;
   var bstr = '<br><input type="button" value="Values" onclick="DisplayValues(this);"><br>';

   str = 'Something long to scroll with style "' + args.style + '":<br><br>';
   str += '<div style="' + args.style + '">';

   for (i=0; i<args.size; i++) {
      if (i % args.button == 0) str += bstr;
      str += i + ": The quick brown fox jumps over the lazy dog's back. ";
      }

   str += bstr;

   str += '</div>';

   trele.innerHTML = str;

   }


//
// SpeedTest(trele, args) - Runs some speed tests and displays the results.
//
// Control number with args.ntimes.
//

function SpeedTest(trele, args) {

   var str, i, j, regex, val, n10;

   str = '<i>You should run this more than once.</i><br><br><b>Speed Tests (in seconds):</b><br><br>';

   var fullstart = new Date();

   str += 'Tested at: ' + Date() + '<br><br>';

   var starttime;

   var n = args.ntimes || 10000;
   n10 = n/10;

   starttime = StartTiming();
   var objtest = {};
   for (i=0; i<n; i++) {
      j = "N"+i;
      }

   str += 'Doing j="N"+i ' + AddCommas(n) + ' times: ' + StopTiming(starttime) + '<br>';

   starttime = StartTiming();
   var objtest = {};
   for (i=0; i<n; i++) {
      objtest["N"+i] = i;
      }

   str += 'Creating ' + AddCommas(n) + ' object properties (and their names): ' + StopTiming(starttime) + '<br>';

   starttime = StartTiming();
   j = 0;
   for (i=0; i<n; i++) {
      j += objtest["N1000"];
      }

   str += 'Accessing an object property ' + AddCommas(n) + ' times: ' + StopTiming(starttime) + '<br>';

   starttime = StartTiming();
   var arraytest = [];
   for (i=0; i<n; i++) {
      arraytest.push(i);
      }

   str += 'Pushing ' + AddCommas(n) + ' new array values: ' + StopTiming(starttime) + '<br>';

   starttime = StartTiming();
   j = 0;
   for (i=0; i<n; i++) {
      j += arraytest[i];
      }

   str += 'Summing ' + AddCommas(n) + ' array number values: ' + StopTiming(starttime) + '<br>';

   starttime = StartTiming();
   j = "abcdefghijklmnopqrstuvwxyz";
   regex = /m.*q/;
   for (i=0; i<n10; i++) {
      val = j.match(regex);
      }

   str += 'Doing ' + AddCommas(n10) + ' regular expression matches: ' + StopTiming(starttime) + '<br>';

   str += '<br>Total time: ' + StopTiming(fullstart) + '<br>';

   trele.innerHTML = str;

   }


//
// EventTest(trele, args) - Makes a div and shows events it gets.
//

function EventTest(trele, args) {

   var str;

   str = '<div id="ediv" style="border:3px solid black;width:100px;height:100px;background-color:gray;">&nbsp;</div>';
   str += '<br><br><input type="button" value="Clear" onclick="ResetEventList();"><br>';
   str += '<div id="msgdiv">Messages:<br></div>';

   trele.innerHTML = str;

   var ele = GEBI("ediv");

   ele.onmousedown = GotEvent;
   ele.onmousemove = GotEvent;
   ele.onmouseup = GotEvent;
   ele.onfocus = GotEvent;
   ele.onblur = GotEvent;

   }

var EventList = []; // list of events found so far
var LastEventType = ""; // type of last event
var LastEventCount = 0; // how many of last event

//
// GotEvent(e)
//

function GotEvent(e) {

   var ele = GEBI("msgdiv");

   if (!e) {
      e = window.event;
      }

   if (e.type == LastEventType) {
      LastEventCount++;
      EventList[EventList.length-1] = e.type + " event number: " + LastEventCount;
      }
   else {
      LastEventCount = 0;
      LastEventType = e.type;
      EventList.push('New ' + e.type + ' event.');
      }

   ele.innerHTML = 'Messages:<br>' + EventList.join('<br>');

   }


//
// ResetEventList(e)
//

function ResetEventList(e) {

   var ele = GEBI("msgdiv");

   ele.innerHTML='Messages:<br>';
   LastEventType = "";
   EventList = [];

   }

//
// DisplayValues(ele)
//

function DisplayValues(ele) {

   var str = "";

   var elepos = GetElementPosition(ele);

   str += "Left: " + elepos.left + " , Top: " + elepos.top + "\n";

   var viewinfo = GetViewportInfo();

   str += "Width: " + viewinfo.width + ", Height: " + viewinfo.height + "\n";
   str += "Horiz Scroll Offset: " + viewinfo.horizontalScroll + ", Vert Scroll Offset: " + viewinfo.verticalScroll;

   alert(str);

   }

//
// ErrorHandler
//
// Flanagan, JavaScript, 5th Edition, page 289
//

function ErrorHandler(msg, url, line) {

   alert("ERROR: " + msg + "\n" + line);

   return true;

   }

// GEBI(id) - returns element object

function GEBI(id) {
   return document.getElementById(id);
   }

//
// GetViewportInfo() - returns object with viewport width and height, and scroll offsets
//
// Flanagan, JavaScript, 5th Edition, page 276
//

function GetViewportInfo() {

   var result = {};

   if (window.innerWidth) { // all but IE
      result.width = window.innerWidth;
      result.height = window.innerHeight;
      result.horizontalScroll = window.pageXOffset;
      result.verticalScroll = window.pageYOffset;
      }
   else {
      if (document.documentElement && document.documentElement.clientWidth) {
         result.width = document.documentElement.clientWidth;
         result.height = document.documentElement.clientHeight;
         result.horizontalScroll = document.documentElement.scrollLeft;
         result.verticalScroll = document.documentElement.scrollTop;
         }
      else if (document.body.clientWidth) {
         result.width = document.body.clientWidth;
         result.height = document.body.clientHeight;
         result.horizontalScroll = document.body.scrollLeft;
         result.verticalScroll = document.body.scrollTop;
         }
      }

   return result;
   }

//
// GetElementPosition(element) - returns object with left and top position of the element in the document
//
// Goodman's JavaScript & DHTML Cookbook, 2nd Edition, page 415
//

function GetElementPosition(element) {

   var offsetLeft = 0;
   var offsetTop = 0;
   while (element) {
      offsetLeft+=element.offsetLeft;
      offsetTop+=element.offsetTop;
      element=element.offsetParent;
      }
   return {left:offsetLeft, top:offsetTop};

   }


// StartTiming() - returns a value to use for timing

function StartTiming() {

   return new Date();

   }

// StopTiming(starttime) - returns the number of seconds since timing was started

function StopTiming(starttime) {

   return ((new Date()) - starttime)/1000;

   }

// AddCommas(n) - returns the number as a string with thousand separators

function AddCommas(n) {

   var str = n + "";

   str = str.replace(/(\d)(\d\d\d)$/g, "$1,$2");
   str = str.replace(/(\d)(\d\d\d)(,\d\d\d)$/g, "$1,$2$3");

   return str;

   }

